mod file_tab;
mod main_tab;

use crate::MainWindow;
use file_tab::{FileTab, SharedFileTab};
use main_tab::{MainTab, SharedMainTab};
use rfd::MessageDialog;
use std::{cell::RefCell, rc::Rc, sync::Mutex};
use slint::{Rgba8Pixel, SharedPixelBuffer};
use tim_tool::logic::TIMManager;
use tiny_skia::{Rect, Transform};

type MainWindowRef  = Rc<RefCell<MainWindow>>;

pub type MutexTIMManager = Rc<Mutex<TIMManager>>;

pub const VRAM_WIDTH:usize  = 1024;
pub const VRAM_HEIGHT:usize = 512;

pub struct GUIElements {
    _file_tab: SharedFileTab,
    _main_tab: SharedMainTab,
}

impl GUIElements {
    pub fn new(main_window: MainWindowRef, tim_manager: MutexTIMManager) -> Result<GUIElements, slint::PlatformError> {
        main_window.borrow().set_main_tab_vram_bg(GUIElements::create_bg()?);

        let _main_tab = MainTab::new(main_window.clone(), tim_manager.clone());
        let _file_tab = FileTab::new(main_window.clone(), _main_tab.clone(), tim_manager);
        
        Ok(GUIElements{_file_tab, _main_tab})
    }

    fn create_bg() -> Result<slint::Image, slint::PlatformError> {
        Ok(slint::Image::from_rgba8_premultiplied(create_vram_bg(320, 256).ok_or(slint::PlatformError::Other("Failed creating VRAM background image".to_string()))?))
    }
}


pub fn display_information(title: impl Into<String>, text: impl Into<String>) {
    MessageDialog::new().set_title(title).set_level(rfd::MessageLevel::Info).set_description(text).show();
}

pub fn display_error(title: impl Into<String>, text: impl Into<String>) {
    MessageDialog::new().set_title(title).set_level(rfd::MessageLevel::Error).set_description(text).show();
}

fn create_vram_bg(rect_width: u32, rect_height: u32) -> Option<SharedPixelBuffer<Rgba8Pixel>> {
    let mut pixel_buffer = SharedPixelBuffer::<Rgba8Pixel>::new(VRAM_WIDTH as u32, VRAM_HEIGHT as u32);
    let     width        = pixel_buffer.width();
    let     height       = pixel_buffer.height();

    if let Some(mut pixmap) = tiny_skia::PixmapMut::from_bytes(pixel_buffer.make_mut_bytes(), width, height) {
        let vram_color           = tiny_skia::Color::from_rgba8(0x0,  0x80, 0x80, 0xFF); 
        let display_buffer_color = tiny_skia::Color::from_rgba8(0x0,  0x80,  0x0, 0xFF);
        let draw_area_color      = tiny_skia::Color::from_rgba8(0x80, 0x80,  0x0, 0xFF);

        let mut paint = tiny_skia::Paint::default();

        pixmap.fill(vram_color);
        
        paint.set_color(display_buffer_color);
        pixmap.fill_rect(Rect::from_xywh(0.0, 0.0,                rect_width as f32, rect_height as f32).unwrap(), &paint, Transform::identity(), None);
        paint.set_color(draw_area_color);
        pixmap.fill_rect(Rect::from_xywh(0.0, rect_height as f32, rect_width as f32, rect_height as f32).unwrap(), &paint, Transform::identity(), None);
        Some(pixel_buffer)
    }
    
    else {
        None
    }
}